@@ -22,10 +22,11 @@
 
 import os
 import gi
-gi.require_version("Gdk", "3.0")
-gi.require_version("Gtk", "3.0")
+gi.require_version("Gdk", "4.0")
+gi.require_version("Gtk", "4.0")
+gi.require_version("Adw", "1")
 gi.require_version("PackageKitGlib", "1.0")
-from gi.repository import GObject, Gdk, Gtk
+from gi.repository import Gtk, Adw
 from gi.repository import PackageKitGlib as packagekit
 from gettext import gettext as _
 
@@ -40,41 +41,42 @@ class ProgressDialog(Gtk.Window):
     def __init__(self, parent):
         Gtk.Window.__init__(self, title=_("Cache Refresh"))
         self.set_transient_for(parent)
-        self.set_position(Gtk.WindowPosition.CENTER)
-        self.set_border_width(16)
         self.set_modal(True)
         self.set_deletable(False)
 
-        self.set_default_size(300, 75)
-        geometry = Gdk.Geometry()
-        geometry.min_width = 210
-        geometry.min_height = 60
-        geometry.max_width = 800
-        geometry.max_height = 260
-        self.set_geometry_hints(None, geometry, Gdk.WindowHints.MIN_SIZE)
-        self.set_geometry_hints(None, geometry, Gdk.WindowHints.MAX_SIZE)
-
         self.box = Gtk.Box(spacing=6, orientation=Gtk.Orientation.VERTICAL)
-        self.add(self.box)
+        self.box.set_margin_top(12)
+        self.box.set_margin_bottom(12)
+        self.box.set_margin_start(12)
+        self.box.set_margin_end(12)
+        self.set_child(self.box)
 
         self.label = Gtk.Label(xalign=0)
         self.label.set_markup("<b><big>{}</big></b>".format(_("Refreshing software cache")))
-        self.box.pack_start(self.label, False, False, 0)
+        self.box.append(self.label)
 
         # create a progress bar
         self.progressbar = Gtk.ProgressBar()
-        self.box.pack_start(self.progressbar, True, True, 0)
+        self.box.append(self.progressbar)
 
 
 class DialogCacheOutdated:
-    def __init__(self, parent, datadir):
+    def __init__(self, parent, on_done, datadir):
         """setup up the gtk dialog"""
         self.parent = parent
+        self.on_done = on_done
 
         setup_ui(self, os.path.join(datadir, "gtkbuilder", "dialog-cache-outofdate.ui"), domain="software-properties")
         self.dialog = self.dialog_cache_outofdate
         self.dialog.set_transient_for(parent)
 
+    def _report_exception_and_finish(self, message, detail):
+        d = Adw.MessageDialog.new(self.parent, str(message), str(detail))
+        d.add_response("ok", _("_OK"))
+        d.set_close_response("ok")
+        d.connect("response", lambda d, r: (self.on_done() if callable(self.on_done) else None))
+        d.present()
+
     def on_pktask_progress(self, progress, ptype, udata=(None,)):
         if ptype == packagekit.ProgressType.PERCENTAGE:
             perc = progress.get_property('percentage')
@@ -84,35 +86,42 @@ class DialogCacheOutdated:
         try:
             self._pktask.generic_finish(result)
         except Exception as e:
-            dialog = Gtk.MessageDialog(self.parent, 0, Gtk.MessageType.ERROR,
-                Gtk.ButtonsType.CANCEL, _("Error while refreshing cache"))
-            dialog.format_secondary_text(str(e))
-            dialog.run()
-        self._loop.quit ()
-
-    def run(self):
-        """run the dialog, and if reload was pressed run cache update"""
-        res = self.dialog.run()
-        self.dialog.hide()
-        if res == Gtk.ResponseType.APPLY:
-            self._pktask = packagekit.Task()
-            self._pdia = ProgressDialog(self.parent)
-            self._loop = GObject.MainLoop()
-            self._pdia.show_all()
-
-            self.parent.set_sensitive(False)
-            try:
-                self._pktask.refresh_cache_async (False, # force
-                                                  None,  # GCancellable
-                                                  self.on_pktask_progress,
-                                                  (None,), # user data
-                                                  self.on_pktask_finish,
-                                                  (None,));
-            except Exception as e:
-                print("Error while requesting cache refresh: {}".format(e))
-
-            self._loop.run()
             self._pdia.hide()
             self.parent.set_sensitive(True)
+            self._report_exception_and_finish(_("Error while refreshing cache"), str(e))
+            return
 
-        return res
+        self._pdia.hide()
+        self.parent.set_sensitive(True)
+        if callable(self.on_done):
+            self.on_done()
+
+    def present(self):
+        def _on_response(dialog, response_id):
+            dialog.hide()
+            if response_id == Gtk.ResponseType.APPLY:
+                self._pktask = packagekit.Task()
+                self._pdia = ProgressDialog(self.parent)
+                self._pdia.present()
+                self.parent.set_sensitive(False)
+                try:
+                    self._pktask.refresh_cache_async(
+                        False,              # force
+                        None,               # cancellable
+                        self.on_pktask_progress,
+                        (None,),            # progress data
+                        self.on_pktask_finish,
+                        (None,),            # finish data
+                    )
+                except Exception as e:
+                    self._pdia.hide()
+                    self.parent.set_sensitive(True)
+                    self._report_exception_and_finish(_("Error while refreshing cache"), str(e))
+            else:
+                # User chose to close without reload; proceed to quit
+                if callable(self.on_done):
+                    self.on_done()
+
+        self.dialog.set_modal(True)
+        self.dialog.connect("response", _on_response)
+        self.dialog.present()
