@@ -20,9 +20,9 @@ import dbus
 import os
 from gettext import gettext as _
 import gi
-gi.require_version("Gtk", "3.0")
-from gi.repository import GdkPixbuf, Gio, Gtk
-from softwareproperties.gtk.utils import current_distro, is_dark_theme
+gi.require_version("Gtk", "4.0")
+from gi.repository import Gio, Adw
+from softwareproperties.gtk.utils import current_distro
 
 from .DialogUaAttach import DialogUaAttach
 from .DialogUaDetach import DialogUaDetach
@@ -77,11 +77,13 @@ class UbuntuProPage(object):
         self.label_ua_usg_status = parent.label_ua_usg_status
         self.label_ua_usg_description = parent.label_ua_usg_description
 
-        if is_dark_theme(self.stack_ua_attach):
-            ubuntu_pro_logo = GdkPixbuf.Pixbuf.new_from_file_at_scale(os.path.join(parent.datadir, 'ubuntu-pro-logo-dark.svg'), -1, 50, True)
-        else:
-            ubuntu_pro_logo = GdkPixbuf.Pixbuf.new_from_file_at_scale(os.path.join(parent.datadir, 'ubuntu-pro-logo.svg'), -1, 50, True)
-        parent.image_ubuntu_pro_logo.set_from_pixbuf(ubuntu_pro_logo)
+        # TODO: This doesn't switch the logo dynamically based on dark/light mode.
+        logo_path = os.path.join(
+            parent.datadir,
+            'ubuntu-pro-logo-dark.svg' if Adw.StyleManager.get_default().get_dark() else 'ubuntu-pro-logo.svg',
+        )
+        parent.picture_ubuntu_pro_logo.set_filename(logo_path)
+        parent.picture_ubuntu_pro_logo.set_size_request(200, 50)
 
         # Display the tab on init which is only on LTS series
         self.stack_ua_main.set_visible(True)
@@ -219,17 +221,18 @@ class UbuntuProPage(object):
 
     def on_button_ua_attach_clicked(self, button):
         dialog = DialogUaAttach(self._parent.window_main, self._parent.datadir, self.ua_object)
-        dialog.run()
+        dialog.present()
 
     def on_button_ua_detach_clicked(self, button):
         dialog = DialogUaDetach(self._parent.window_main, self._parent.datadir, self.ua_object)
-        dialog.run()
+        dialog.present()
 
     def set_service_enabled(self, service_name, enabled, error_label, error_label_messages):
         if error_label is not None:
             error_label.set_visible(False)
         service = self.get_service(service_name)
         if service is None:
+            # TODO: Add some error reporting
             return
         def on_reply():
             service.request_in_progress = False
@@ -272,23 +275,23 @@ class UbuntuProPage(object):
 
     def on_button_ua_fips_clicked(self, button):
         dialog = DialogUaFipsEnable(self._parent.window_main, self._parent.datadir, self.ua_object)
-        service_name = dialog.run()
-        if service_name is None:
-            return
-
-        dialog = Gtk.MessageDialog(parent=self._parent.window_main,
-                                   flags=Gtk.DialogFlags.MODAL,
-                                   type=Gtk.MessageType.QUESTION,
-                                   message_format=None)
-        dialog.add_button(_('No, go back'), Gtk.ResponseType.CANCEL)
-        dialog.add_button(_('Enable FIPS'), Gtk.ResponseType.OK)
-        dialog.set_markup(_('Enabling FIPS could take a few minutes. This action cannot be reversed. Are you sure you want to enable FIPS?'))
-        result = dialog.run()
-        dialog.destroy()
-        if result != Gtk.ResponseType.OK:
-            return
-
-        self.set_service_enabled(service_name, True, None, None)
+        def _on_fips_choice(service_name):
+            if service_name is None:
+                return
+            confirm = Adw.MessageDialog.new(
+                self._parent.window_main,
+                _('Enable FIPS'),
+                _('Enabling FIPS could take a few minutes. This action cannot be reversed. Are you sure you want to enable FIPS?')
+            )
+            confirm.add_response('cancel', _('No, go back'))
+            confirm.add_response('ok', _('Enable FIPS'))
+            confirm.set_close_response('cancel')
+            def _on_confirm_response(_dlg, response):
+                if response == 'ok':
+                    self.set_service_enabled(service_name, True, None, None)
+            confirm.connect('response', _on_confirm_response)
+            confirm.present()
+        dialog.present(_on_fips_choice)
 
     def on_button_ua_usg_clicked(self, button):
         service = self.get_service('usg')
@@ -296,4 +299,4 @@ class UbuntuProPage(object):
         self.set_service_enabled('usg', not is_enabled, None, None)
 
     def on_compliance_and_hardening_expand_changed(self, widget, param):
-        self._parent.window_main.resize(1, 1)
+        self._parent.window_main.set_default_size(1, 1)
