@@ -21,9 +21,8 @@ from datetime import datetime
 import distro_info
 from functools import wraps
 import gi
-gi.require_version("Gtk", "3.0")
-gi.require_version("Handy", "1")
-from gi.repository import Gio, Gtk, GLib, Handy
+gi.require_version("Gtk", "4.0")
+from gi.repository import Gio, Gtk, GObject
 import json
 import os
 import subprocess
@@ -38,22 +37,19 @@ UA_STATUS_JSON = "/var/lib/ubuntu-advantage/status.json"
 def setup_ui(self, path, domain):
     # setup ui
     self.builder = Gtk.Builder()
+    # In GTK4, set_current_object requires a GObject; dialogs here may be plain Python classes
+    if isinstance(self, GObject.Object):
+        self.builder.set_current_object(self)
     self.builder.set_translation_domain(domain)
     self.builder.add_from_file(path)
-    self.builder.connect_signals(self)
     for o in self.builder.get_objects():
         if issubclass(type(o), Gtk.Buildable):
-            name = Gtk.Buildable.get_name(o)
-            setattr(self, name, o)
+            setattr(self, o.get_buildable_id(), o)
         else:
-            logging.debug("can not get name for object '%s'" % o)
+            logging.debug(f"can not get name for object '{o}'")
 
 def has_gnome_online_accounts():
-    try:
-        d = Gio.DesktopAppInfo.new('gnome-online-accounts-panel.desktop')
-        return d != None
-    except Exception:
-        return False
+    return Gio.DesktopAppInfo.new('gnome-online-accounts-panel.desktop') != None
 
 def is_current_distro_lts():
     distro = aptsources.distro.get_distro()
@@ -179,9 +175,3 @@ def retry(exceptions, tries=10, delay=0.1, backoff=2):
         return f_retry  # true decorator
 
     return deco_retry
-
-def is_dark_theme(widget):
-    env_gtk_theme = GLib.getenv("GTK_THEME")
-    if env_gtk_theme != None:
-        return GLib.str_has_suffix(env_gtk_theme, "dark")
-    return Handy.StyleManager.get_default().get_dark()
